@@ -16,15 +16,15 @@ use List::Util 1.33 ();
 use Type::Coercion;
 use Type::Tiny 1.002001;
 use Type::Utils -all;
-use Types::Standard qw/ -types is_ArrayRef is_HashRef is_Ref /;
+use Types::Standard qw/ -types is_ArrayRef is_HashRef is_ScalarRef /;
 use Types::TypeTiny ();
 
-# RECOMMEND PREREQ: Ref::Util::XS
+# RECOMMEND PREREQ: Ref::Util::XS 0.100
 # RECOMMEND PREREQ: Type::Tiny::XS
 
 use namespace::autoclean 0.28;
 
-our $VERSION = 'v0.3.0';
+our $VERSION = 'v0.3.2';
 
 
 declare Const,
@@ -47,21 +47,20 @@ sub __coerce_constant {
 }
 
 sub __is_readonly {
-    if ( is_ArrayRef($_) ) {
-        return Internals::SvREADONLY(@$_)
-          && List::Util::all { __is_readonly($_) } @$_;
+    if ( is_ArrayRef( $_[0] ) ) {
+        return Internals::SvREADONLY( @{ $_[0] } )
+          && List::Util::all { __is_readonly($_) } @{ $_[0] };
     }
-    elsif ( is_HashRef($_) ) {
-        &Internals::hv_clear_placeholders($_);
-        return Internals::SvREADONLY(%$_)
-          && List::Util::all { __is_readonly($_) } values %$_;
+    elsif ( is_HashRef( $_[0] ) ) {
+        &Internals::hv_clear_placeholders( $_[0] );
+        return Internals::SvREADONLY( %{ $_[0] } )
+          && List::Util::all { __is_readonly($_) } values %{ $_[0] };
     }
-    elsif ( is_Ref($_) ) {
-        return Internals::SvREADONLY($$_);
-    }
-    else {
-        return Internals::SvREADONLY($_);
+    elsif ( is_ScalarRef( $_[0] )  ) {
+        return Internals::SvREADONLY( ${ $_[0] } );
     }
+
+    return Internals::SvREADONLY( $_[0] );
 }
 
 sub __constraint_generator {
@@ -119,7 +118,7 @@ Types::Const - Types that coerce references to read-only
 
 =head1 VERSION
 
-version v0.3.0
+version v0.3.2
 
 =head1 SYNOPSIS
 
@@ -131,7 +130,7 @@ version v0.3.0
 
   has bar => (
     is      => 'ro',
-    isa     => ConstArrayRef[Str],
+    isa     => Const[ArrayRef[Str]],
     coerce  => 1,
   );
 
@@ -176,6 +175,21 @@ be thrown if you attempt to fetch the value of a non-existent key:
 
 The work around for this is to check that a key exists beforehand.
 
+=head2 Performance issues
+
+Validating that a complex data-structure is read-only can affect
+performance.  If this is an issue, one workaround is to use
+L<Devel::StrictMode> and only validate data structures during tests:
+
+  has bar => (
+    is      => 'ro',
+    isa     => STRICT ? Const[ArrayRef[Str]] : ArrayRef,
+    coerce  => 1,
+  );
+
+Another means of improving performance is to only check the type
+once. (Since it is read-only, there is no need to re-check it.)
+
 =head2 Bug reports and feature requests
 
 Please report any bugs or feature requests on the bugtracker website
